# Stolen from
# https://github.com/ryutok/mpl_axes_aligner


def _calc_rorg(org, ival, fval):
    rorg = (org - ival) / (fval - ival)
    if rorg < 0:
        rorg = 0
        ival = org
    elif rorg > 1:
        rorg = 1
        fval = org
    return rorg, ival, fval


def _calc_pos(org1, org2, lim1, lim2):
    ival1, fval1 = lim1
    ival2, fval2 = lim2

    rorg1, ival1, fval1 = _calc_rorg(org1, ival1, fval1)
    rorg2, ival2, fval2 = _calc_rorg(org2, ival2, fval2)
    pos = (rorg1 + rorg2) / 2

    if pos == 0 or pos == 1:
        raise ValueError("When pos=None, at least one origin should be "
                         "within the initial plotting range.")

    return pos


def _expand_range(org, pos, ival, fval):
    if pos <= 0 or pos >= 1:
        raise ValueError("When expand=True, the position to align the origin "
                         "should be 0 < pos < 1.")

    rorg = (org - ival) / (fval - ival)
    if rorg > pos:
        fval = (org - ival + pos*ival) / pos
    else:
        ival = (org - pos*fval) / (1 - pos)
    return ival, fval


def shiftyaxis(ax, org, pos, expand=False):
    bottom, top = ax.get_ylim()
    bottom, top = _expand_range(org, pos, bottom, top)
    ax.set_ylim(bottom, top)


def yaxes(ax1, org1, ax2, org2):
    # Get plotting ranges
    try:
        lim1 = list(ax1.get_ylim())
        lim2 = list(ax2.get_ylim())
    except AttributeError or TypeError:
        raise TypeError("'ax1' and 'ax2' should be Axes objects of "
                        "matplotlib.")

    # Calculate the position
    pos = _calc_pos(org1, org2, lim1, lim2)

    # Apply the new ranges
    shiftyaxis(ax1, org1, pos, True)
    shiftyaxis(ax2, org2, pos, True)
